@@ -19,7 +19,7 @@ from scholarqa.rag.multi_step_qa_pipeline import MultiStepQAPipeline
 from scholarqa.rag.retrieval import PaperFinder
 from scholarqa.state_mgmt.local_state_mgr import AbsStateMgrClient, LocalStateMgrClient
 from scholarqa.trace.event_traces import EventTrace
-from scholarqa.utils import get_paper_metadata
+from scholarqa.utils import get_paper_metadata, NUMERIC_META_FIELDS, CATEGORICAL_META_FIELDS
 from langsmith import traceable
 
 logger = logging.getLogger(__name__)
@@ -100,7 +100,7 @@ class ScholarQA:
         rewritten_query = llm_processed_query.rewritten_query
         keyword_query = llm_processed_query.keyword_query
         self.update_task_state(
-            f"Retrieving relevant passages from 8M+ open access papers",
+            f"Retrieving relevant passages from a corpus of 8M+ open access papers",
             step_estimated_time=5
         )
         # Get relevant paper passages from the Semantic Scholar index for the llm rewritten query
@@ -115,7 +115,7 @@ class ScholarQA:
                                                                               **llm_processed_query.search_filters)
             search_api_results = [item for item in search_api_results if item["corpus_id"] not in snippet_corpus_ids]
             self.update_task_state(
-                f"Retrieved {len(search_api_results)} more papers from Semantic Scholar using keyword search",
+                f"Retrieved {len(search_api_results)} more papers from Semantic Scholar abstracts using keyword search",
                 step_estimated_time=1)
         else:
             search_api_results = []
@@ -158,6 +158,11 @@ class ScholarQA:
         per_paper_summaries = self.llm_caller.call_method(cost_args, self.multi_step_pipeline.step_select_quotes,
                                                           query=query, scored_df=scored_df,
                                                           sys_prompt=sys_prompt)
+        api_corpus_ids = set(
+            scored_df[scored_df.sentences.apply(lambda x: not x)].corpus_id.astype(str))
+        ref_strs = {rs.split(" | ")[0][1:] for rs in per_paper_summaries.result}
+        logger.info(f"Paper abstracts used from s2 api: {api_corpus_ids.intersection(ref_strs)}")
+
         logger.info(
             f"Step 1 done - {len(per_paper_summaries.result)} papers with quotes extracted, cost: {per_paper_summaries.tot_cost}, "
             f"time: {time() - start:.2f}")
@@ -178,17 +183,11 @@ class ScholarQA:
         return cluster_json
 
     @traceable(name="Generation: Generate an iterative summary")
-    def step_gen_iterative_summary(self, query: str, per_paper_summaries: Dict[str, str], sec_names: List[str],
+    def step_gen_iterative_summary(self, query: str, per_paper_summaries: Dict[str, str],
                                    plan_json: Dict[str, Any], cost_args: CostReportingArgs,
                                    sys_prompt: str = PROMPT_ASSEMBLE_SUMMARY) -> Generator[
         str, None, CostAwareLLMResult]:
         logger.info("Running Step 3: Assemble the summary with the links (takes ~2 mins)")
-        task_estimated_time = 30 + 15 * len(plan_json)
-        task_estimated_time = max((task_estimated_time + task_estimated_time % 60) // 60, 1)
-        outline = '\n    - ' + '\n    - '.join(sec_names)
-        self.update_task_state(f"Start generating each section in the answer outline: {outline}",
-                               task_estimated_time=f"~{task_estimated_time} minutes" if task_estimated_time > 1 else "~1 minute",
-                               step_estimated_time=15)
         start = time()
 
         cost_args = cost_args._replace(model=self.multi_step_pipeline.llm_model)._replace(
@@ -291,9 +290,12 @@ class ScholarQA:
         event_trace.trace_retrieval_event(retrieved_candidates)
 
         # Rerank the retrieved candidates based on the query with a cross encoder
+        s2_srch_metadata = [{k: v for k, v in paper.items() if
+                             k == "corpus_id" or k in NUMERIC_META_FIELDS or k in CATEGORICAL_META_FIELDS} for paper in
+                            s2_srch_res]
         reranked_df, paper_metadata = self.rerank_and_aggregate(query, retrieved_candidates,
                                                                 {str(paper["corpus_id"]): paper for paper in
-                                                                 s2_srch_res})
+                                                                 s2_srch_metadata})
         if reranked_df.empty:
             raise Exception(
                 "No relevant papers found for the query post reranking, skipping quote extraction.")
@@ -321,26 +323,36 @@ class ScholarQA:
         event_trace.trace_inline_citation_following_event(per_paper_summaries_extd)
 
         # step 3: generating output as per the outline
+        section_titles = [dim["name"] for dim in cluster_json.result["dimensions"]]
         gen_sections_iter = self.step_gen_iterative_summary(query, per_paper_summaries_extd,
-                                                            [dim["name"] for dim in cluster_json.result["dimensions"]],
                                                             plan_json, cost_args)
 
         json_summary, generated_sections, table_threads = [], [], []
         tables = [None for _ in cluster_json.result["dimensions"]]
         citation_ids = dict()
 
+        task_estimated_time = 30 + 15 * len(plan_json)
+        task_estimated_time = max((task_estimated_time + task_estimated_time % 60) // 60, 1)
+        outline = '\n    - ' + '\n    - '.join(section_titles)
+        self.update_task_state(f"Start generating each section in the answer outline: {outline}",
+                               task_estimated_time=f"~{task_estimated_time} minutes" if task_estimated_time > 1 else "~1 minute",
+                               step_estimated_time=15)
+
         try:
-            idx_iter = enumerate(gen_sections_iter)
+            gen_iter = gen_sections_iter
+            idx = 0
             while True:
-                idx, section_text = next(idx_iter)
+                if idx < len(plan_json):
+                    self.update_task_state(
+                        f"Iteratively generating section: {(idx + 1)} of {len(plan_json)} - {section_titles[idx]}",
+                        curr_response=generated_sections, step_estimated_time=15)
+                section_text = next(gen_iter)
                 section_json = \
                     get_json_summary(self.multi_step_pipeline.llm_model, [section_text], per_paper_summaries_extd,
                                      paper_metadata,
                                      citation_ids, inline_tags)[0]
                 section_json["format"] = cluster_json.result["dimensions"][idx]["format"]
-                self.update_task_state(
-                    f"Iteratively generating section: {(idx + 1)} of {len(plan_json)} - {section_json.get('title', '')}",
-                    curr_response=generated_sections, step_estimated_time=15)
+
                 json_summary.append(section_json)
                 self.postprocess_json_output(json_summary)
                 if section_json["format"] == "list" and section_json["citations"]:
@@ -352,6 +364,7 @@ class ScholarQA:
                         table_threads.append(tthread)
                 gen_sec = self.get_gen_sections_from_json(section_json)
                 generated_sections.append(gen_sec)
+                idx += 1
         except StopIteration as e:
             all_sections = e.value
 
